#include <stdio.h>
#include <xtables.h>
#include "xt_comm.h"

static void comm_help(void)
{
	printf(
"comm match options:\n"
"[!] --comm command   Match command name\n");
}

static const struct xt_option_entry comm_opts[] = {
	{.name = "comm", .type = XTTYPE_STRING,
	 .flags = XTOPT_INVERT | XTOPT_MAND | XTOPT_PUT,
	 XTOPT_POINTER(struct xt_comm_info, comm)},
	XTOPT_TABLEEND,
};

static void comm_parse(struct xt_option_call *cb)
{
	struct xt_comm_info *comminfo = cb->data;

	xtables_option_parse(cb);
	if (cb->invert)
		comminfo->invert = true;
}

static void
comm_print(const void *ip, const struct xt_entry_match *match, int numeric)
{
	const struct xt_comm_info *info = (void *)match->data;

	printf(" comm %s%s", info->invert ? "! ":"", info->comm);
}

static void comm_save(const void *ip, const struct xt_entry_match *match)
{
	const struct xt_comm_info *info = (void *)match->data;

	printf("%s --comm %s", info->invert ? " !" : "", info->comm);
}

static int comm_xlate(struct xt_xlate *xl,
		     const struct xt_xlate_mt_params *params)
{
	const struct xt_comm_info *info = (void *)params->match->data;

	xt_xlate_add(xl, "comm%s %s", info->invert ? " !=" : "", info->comm);

	return 1;
}

static struct xtables_match comm_match = {
	.family		= NFPROTO_UNSPEC,
	.name		= "comm",
	.version	= XTABLES_VERSION,
	.size		= XT_ALIGN(sizeof(struct xt_comm_info)),
	.userspacesize	= XT_ALIGN(sizeof(struct xt_comm_info)),
	.help		= comm_help,
	.print		= comm_print,
	.save		= comm_save,
	.x6_parse	= comm_parse,
	.x6_options	= comm_opts,
	.xlate		= comm_xlate,
};

void _init(void)
{
	xtables_register_match(&comm_match);
}
